"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HapifySyntax = exports.TimeoutError = exports.ParsingError = exports.InternalError = exports.EvaluationError = exports.ArgumentsError = void 0;
const Hoek = __importStar(require("@hapi/hoek"));
const vm_1 = require("@hapify/vm");
const line_column_1 = __importDefault(require("line-column"));
const ArgumentsError_1 = require("./errors/ArgumentsError");
Object.defineProperty(exports, "ArgumentsError", { enumerable: true, get: function () { return ArgumentsError_1.ArgumentsError; } });
const EvaluationError_1 = require("./errors/EvaluationError");
Object.defineProperty(exports, "EvaluationError", { enumerable: true, get: function () { return EvaluationError_1.EvaluationError; } });
const InternalError_1 = require("./errors/InternalError");
Object.defineProperty(exports, "InternalError", { enumerable: true, get: function () { return InternalError_1.InternalError; } });
const ParsingError_1 = require("./errors/ParsingError");
Object.defineProperty(exports, "ParsingError", { enumerable: true, get: function () { return ParsingError_1.ParsingError; } });
const TimeoutError_1 = require("./errors/TimeoutError");
Object.defineProperty(exports, "TimeoutError", { enumerable: true, get: function () { return TimeoutError_1.TimeoutError; } });
const CommentPattern_1 = require("./patterns/CommentPattern");
const ConditionalPattern_1 = require("./patterns/ConditionalPattern");
const EscapeBackSlashesPattern_1 = require("./patterns/EscapeBackSlashesPattern");
const EscapePattern_1 = require("./patterns/EscapePattern");
const EscapeQuotesPattern_1 = require("./patterns/EscapeQuotesPattern");
const EvaluatePattern_1 = require("./patterns/EvaluatePattern");
const IndentPattern_1 = require("./patterns/IndentPattern");
const InterpolationPattern_1 = require("./patterns/InterpolationPattern");
const IterationPattern_1 = require("./patterns/IterationPattern");
const MetaInterpolationPattern_1 = require("./patterns/MetaInterpolationPattern");
const NameInterpolationPattern_1 = require("./patterns/NameInterpolationPattern");
const NotesInterpolationPattern_1 = require("./patterns/NotesInterpolationPattern");
/** Ordered patterns */
const PatternsStack = [
    IndentPattern_1.IndentPattern,
    EscapeBackSlashesPattern_1.EscapeBackSlashesPattern,
    EscapeQuotesPattern_1.EscapeQuotesPattern,
    CommentPattern_1.CommentPattern,
    ConditionalPattern_1.ConditionalPattern,
    IterationPattern_1.IterationPattern,
    InterpolationPattern_1.InterpolationPattern,
    NameInterpolationPattern_1.NameInterpolationPattern,
    MetaInterpolationPattern_1.MetaInterpolationPattern,
    NotesInterpolationPattern_1.NotesInterpolationPattern,
    EvaluatePattern_1.EvaluatePattern,
    EscapePattern_1.EscapePattern,
];
const DefaultOptions = {
    timeout: 1000,
};
/** @type {HapifySyntax} Syntax parser */
class HapifySyntax {
    /** Constructor */
    constructor(template, model, options = {}) {
        this.template = template;
        this.model = model;
        this.actions = [];
        this.patterns = PatternsStack.map((Pattern) => new Pattern(this));
        this.original = template;
        this.options = Hoek.applyToDefaults(DefaultOptions, options);
    }
    /** Parser method */
    static run(template, model, options = {}) {
        // Check how many arguments
        if (arguments.length < 2) {
            throw new ArgumentsError_1.ArgumentsError('[HapifySyntax.run] Requires at least two arguments');
        }
        // Check arguments
        if (typeof template !== 'string') {
            throw new ArgumentsError_1.ArgumentsError('[HapifySyntax.run] template must be a string');
        }
        if (typeof model !== 'object') {
            throw new ArgumentsError_1.ArgumentsError('[HapifySyntax.run] model must be an object');
        }
        if (model === null) {
            throw new ArgumentsError_1.ArgumentsError('[HapifySyntax.run] model cannot be null');
        }
        const runner = new HapifySyntax(template, model, options);
        // Execute all patterns
        // @todo Should catch parsing error
        runner.parse();
        return runner.evaluate();
    }
    /** Execute all patterns to convert hpf to js */
    parse() {
        for (const pattern of this.patterns) {
            pattern.execute();
        }
    }
    /** Eval the generated script */
    evaluate() {
        // eslint-disable-line no-unused-vars
        // Cannot inject object with key root in context.
        const script = `const root = _root; let out = \n\`${this.template}\`\n; return out;`;
        try {
            const result = new vm_1.HapifyVM({ timeout: this.options.timeout }).run(script, { _root: this.model });
            return this.postProcess(result);
        }
        catch (error) {
            if (error.code === 6003) {
                throw new TimeoutError_1.TimeoutError(`Template processing timed out (${this.options.timeout}ms)`);
            }
            if (error.code === 6002) {
                throw this.getReversedActionError(error, -1);
            }
            throw error;
        }
    }
    /** Reverse all action to find the error line and column in the input file */
    getReversedActionError(error, lineOffset = 0) {
        // Get the line and column of the error
        const lineNumber = typeof error.lineNumber === 'number' ? error.lineNumber + lineOffset : 0;
        const columnNumber = typeof error.columnNumber === 'number' ? error.columnNumber : 0;
        let errorIndex = line_column_1.default(this.template).toIndex(lineNumber, columnNumber);
        // Reverse all actions to find the line and column of the error in the input
        this.actions.reverse().forEach((action) => {
            if (errorIndex >= action.index) {
                // The error is impacted only if the error is in or after the action
                if (errorIndex <= action.index + action.after && action.after !== 0) {
                    // If the error is in the action and the action is not a comment, the error is link to that action
                    errorIndex = action.index;
                }
                else {
                    // Else, move the errorIndex
                    errorIndex += action.before - action.after;
                }
            }
        });
        const errorLineColumn = line_column_1.default(this.original).fromIndex(errorIndex);
        // Create the input error
        const evalError = new EvaluationError_1.EvaluationError(error.message);
        evalError.lineNumber = errorLineColumn ? errorLineColumn.line : null;
        evalError.columnNumber = errorLineColumn ? errorLineColumn.col : null;
        evalError.details = `Error: ${evalError.message}. Line: ${evalError.lineNumber}, Column: ${evalError.columnNumber}`;
        return evalError;
    }
    /** Cleanup generated code */
    postProcess(code) {
        // Removes double empty lines
        const doubleLine = /\r?\n\r?\n/g;
        let output = code;
        while (output.match(doubleLine)) {
            output = output.replace(doubleLine, '\n');
        }
        const doubleLineWithSpace = /\r?\n *\r?\n/g;
        output = output.replace(doubleLineWithSpace, '\n\n');
        output = output.replace(doubleLineWithSpace, '\n\n');
        return output;
    }
}
exports.HapifySyntax = HapifySyntax;
//# sourceMappingURL=index.js.map