"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetaInterpolationPattern = void 0;
const escape_string_regexp_1 = __importDefault(require("escape-string-regexp"));
const ParsingError_1 = require("../errors/ParsingError");
const BasePattern_1 = require("./BasePattern");
const NameInterpolationPattern_1 = require("./NameInterpolationPattern");
/** Convert case words for regexp */
const ForRegExp = (r) => r.search.map(escape_string_regexp_1.default).join('|');
/** Meta interpolation pattern */
const RegEx = new RegExp(`<<-\\s*([a-zA-Z_.]+)\\s+([a-zA-Z0-9_]+)\\s+(${NameInterpolationPattern_1.CasesReplacements.map(ForRegExp).join('|')})\\s*>>`, 'g');
/** MetaInterpolation pattern */
class MetaInterpolationPattern extends BasePattern_1.BasePattern {
    /** Parser method */
    execute() {
        this.replace(RegEx, (match, variable, meta, property) => {
            // Get the var
            let jsVariable = variable;
            if (['M', 'Model'].includes(jsVariable))
                jsVariable = 'root';
            else if (['P', 'PrimaryField'].includes(jsVariable))
                jsVariable = 'root.fields.primary';
            // Get the property
            const matchingCase = NameInterpolationPattern_1.CasesReplacements.find((c) => c.search.includes(property));
            if (!matchingCase) {
                throw new ParsingError_1.ParsingError(`[MetaInterpolationPattern.execute] Unknown name property: ${property}`);
            }
            const jsProperty = matchingCase.replace;
            return `\${(${jsVariable}.meta.${meta} || {}).${jsProperty} || ''}`;
        });
    }
}
exports.MetaInterpolationPattern = MetaInterpolationPattern;
//# sourceMappingURL=MetaInterpolationPattern.js.map