"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NameInterpolationPattern = exports.CasesReplacements = void 0;
const escape_string_regexp_1 = __importDefault(require("escape-string-regexp"));
const ParsingError_1 = require("../errors/ParsingError");
const BasePattern_1 = require("./BasePattern");
exports.CasesReplacements = [
    { search: ['aA', 'camel'], replace: 'camel' },
    { search: ['AA', 'pascal'], replace: 'pascal' },
    { search: ['a', 'lower'], replace: 'lower' },
    { search: ['A', 'capital'], replace: 'capital' },
    { search: ['a-a', 'kebab'], replace: 'kebab' },
    { search: ['A-A', 'header'], replace: 'header' },
    { search: ['a_a', 'snake'], replace: 'snake' },
    { search: ['A_A', 'constant'], replace: 'constant' },
    { search: ['aa', 'compact'], replace: 'compact' },
    { search: ['R', 'raw'], replace: 'raw' },
];
/** Convert case words for regexp */
const ForRegExp = (r) => r.search.map(escape_string_regexp_1.default).join('|');
/** Name interpolation pattern */
const RegEx = new RegExp(`<<([a-zA-Z_.]+)\\s+(${exports.CasesReplacements.map(ForRegExp).join('|')})\\s*>>`, 'g');
/** NameInterpolation pattern */
class NameInterpolationPattern extends BasePattern_1.BasePattern {
    /** Parser method */
    execute() {
        this.replace(RegEx, (match, variable, property) => {
            // Get the var
            let jsVariable = variable;
            if (['M', 'Model'].includes(jsVariable))
                jsVariable = 'root';
            else if (['P', 'PrimaryField'].includes(jsVariable))
                jsVariable = 'root.fields.primary';
            // Get the property
            const matchingCase = exports.CasesReplacements.find((c) => c.search.includes(property));
            if (!matchingCase) {
                throw new ParsingError_1.ParsingError(`[NameInterpolationPattern.execute] Unknown name property: ${property}`);
            }
            const jsProperty = matchingCase.replace;
            return `\${${jsVariable}.names.${jsProperty}}`;
        });
    }
}
exports.NameInterpolationPattern = NameInterpolationPattern;
//# sourceMappingURL=NameInterpolationPattern.js.map